////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2018 The University of North Carolina at Chapel Hill.
// All Rights Reserved.
//
// Permission to use, copy, modify and distribute this software and its
// documentation for educational, research and non-profit purposes, without
// fee, and without a written agreement is hereby granted, provided that the
// above copyright notice and the following three paragraphs appear in all
// copies.
//
// IN NO EVENT SHALL THE UNIVERSITY OF NORTH CAROLINA AT CHAPEL HILL BE
// LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE
// USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY
// OF NORTH CAROLINA HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
// THE UNIVERSITY OF NORTH CAROLINA SPECIFICALLY DISCLAIM ANY
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
// PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// NORTH CAROLINA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
// UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The authors may be contacted via:
//
// US Mail: Real-Time Systems Group at UNC
// Department of Computer Science
// Sitterson Hall
// University of N. Carolina
// Chapel Hill, NC 27599-3175
//
// EMail: nemitz@cs.unc.edu; tamert@cs.unc.edu; anderson@cs.unc.edu
//
////////////////////////////////////////////////////////////////////////////////

#ifndef RNLP_H
#define RNLP_H

#include <stdbool.h>
#include <stdint.h>

#include "mem.h"
#include "nesting.h"
#include "pft.h"
#include "ticket.h"

#define NR_CPUS 64
#define NR_RESOURCES 64

// RNLP

typedef struct __rnlp_node_t {
    unsigned long           length;
    resource_mask_t         requested;
    volatile bool           blocked;
    struct __rnlp_node_t*   next;
} rnlp_node_t;

typedef struct
{
    volatile rnlp_node_t*   tail;
    volatile rnlp_node_t*   head;
    resource_mask_t         locked;
    resource_mask_t         unavailable;
    ticket_lock_t           sublock;
} rnlp_lock_t;

void rnlp_init(rnlp_lock_t* lock);

uint64_t *rnlp_lock(rnlp_lock_t* lock, rnlp_node_t* self);
uint64_t *rnlp_unlock(rnlp_lock_t* lock, rnlp_node_t* self);

#endif // RNLP_H
