#!/bin/bash

# TODO: Set the total number of physical cores on the machine
totalcores=36

# TODO: Set the total number of sockets to be used (tested up to 2)
numsockets=2

# TODO: Set the task counts when no lock server or a floating lock server
# is used (go up to totalcores, e.g., counting by 2s)
# Example: ( 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 36 )
taskcounts_no_server=( 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 36 )

# TODO: Set the task counts when a static global lock server is used
# (go up to totalcores-1, e.g., counting by 2s)
# Example: taskcounts=( 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 35 )
taskcounts_sg_server=( 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 35 )

# TODO: Set the task counts when a static local lock server is used
# (go up to totalcores-numsockets, e.g., counting by 2s)
# Example: taskcounts=( 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 )
taskcounts_sl_server=( 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 )

# Derived: the core counts when no lock server or a floating lock server
# is used
corecounts_no_server=(${taskcounts_no_server[@]})

# Derived: the core counts when a static global lock server is used
# (it needs one extra core for the server)
# Example: corecounts=( 3 5 7 9 11 13 15 17 19 21 23 25 27 29 31 33 35 )
corecounts_sg_server=(${taskcounts_sg_server[@]})
for (( i=0; i<${#taskcounts_sg_server[@]}; i++ ))
do
    corecounts_sg_server[$i]=$(( corecounts_sg_server[$i] + 1 ))
done

# Derived: the core counts when a static local lock server is used
# (it needs one extra core *per socket* for the server)
# Example: corecounts=( 3 5 7 9 11 13 15 17 20 22 24 26 28 30 32 34 36 )
corecounts_sl_server=(${taskcounts_sl_server[@]})
for (( i=0; i<${#taskcounts_sl_server[@]}; i++ ))
do
    if [[ ${taskcounts_sl_server[$i]} -lt $((totalcores / numsockets)) ]]
    then
        corecounts_sl_server[$i]=$(( taskcounts_sl_server[$i] + 1 ))
    else
        corecounts_sl_server[$i]=$(( taskcounts_sl_server[$i] + 2 ))
    fi
done

# Test (example):
### 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 36
### 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 36
###
### 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34 35
### 3 5 7 9 11 13 15 17 19 21 23 25 27 29 31 33 35 36
###
### 2 4 6 8 10 12 14 16 18 20 22 24 26 28 30 32 34
### 3 5 7 9 11 13 15 17 20 22 24 26 28 30 32 34 36

# TODO: Test config if you want
test=0
if [[ $test -eq 1 ]]
then
    echo "No or floating lock server - counts should be equal"
    echo ${taskcounts_no_server[@]}
    echo ${corecounts_no_server[@]}

    echo "Static global lock server - need one more core than tasks"
    echo ${taskcounts_sg_server[@]}
    echo ${corecounts_sg_server[@]}

    echo "Static local lock server(s) - need one more core per socket than tasks"
    echo ${taskcounts_sl_server[@]}
    echo ${corecounts_sl_server[@]}
fi

# TODO: Turn on additional print messages while running experiments,
# if you want.
verbose=0
