////////////////////////////////////////////////////////////////////////////////
//
// Copyright 2018 The University of North Carolina at Chapel Hill.
// All Rights Reserved.
//
// Permission to use, copy, modify and distribute this software and its
// documentation for educational, research and non-profit purposes, without
// fee, and without a written agreement is hereby granted, provided that the
// above copyright notice and the following three paragraphs appear in all
// copies.
//
// IN NO EVENT SHALL THE UNIVERSITY OF NORTH CAROLINA AT CHAPEL HILL BE
// LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
// CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE
// USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY
// OF NORTH CAROLINA HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
// DAMAGES.
//
// THE UNIVERSITY OF NORTH CAROLINA SPECIFICALLY DISCLAIM ANY
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
// PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// NORTH CAROLINA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
// UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// The authors may be contacted via:
//
// US Mail: Real-Time Systems Group at UNC
// Department of Computer Science
// Sitterson Hall
// University of N. Carolina
// Chapel Hill, NC 27599-3175
//
// EMail: nemitz@cs.unc.edu; tamert@cs.unc.edu; anderson@cs.unc.edu
//
////////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include <string.h>

#include "cycles.h"
#include "crnlp_general_sgls.h"
#include "nesting.h"
#include "mem.h"
#include "queue.h"

void crnlp_general_sgls_init(crnlp_general_sgls_lock_t* lock)
{
    int i;

    // Initialize resource queues
    for (i = 0; i < RES_COUNT; i++)
    {
        lock->queues[i].head = NULL;
        lock->queues[i].tail = NULL;
    }

    // Initialize the requests
    for (i = 0; i < QUEUE_LENGTH; i++)
    {
        lock->requests[i].requested = 0;
        lock->requests[i].length = 0;
        lock->requests[i].request_id = i;
        lock->requests[i].enabled = 0;
        lock->requests[i].needs_service = 0;
    }

    lock->pending_requests = 0;
}

uint64_t *crnlp_general_sgls_lock(crnlp_general_sgls_lock_t* lock, uint8_t ind)
{
    crnlp_general_sgls_request_t *self;
    int r;

    uint64_t *ret;
    uint64_t start, end;

    ret = (uint64_t *)malloc(sizeof(uint64_t) * 2);
    ret[0] = ret[1] = 0;

    start = get_cycles();

    self = &(lock->requests[ind]);

    self->enabled = 0;
    self->done = 0;

    mb();

    self->needs_service = 1;
    while (self->needs_service != 0)
    {
        cpu_relax();
    }

    end = get_cycles();
    ret[0] += end - start;
    start = get_cycles();

    // Wait until satisfied
    while (self->enabled == 0)
    {
        cpu_relax();
    }

    end = get_cycles();
    ret[1] += end - start;

    return ret;
}

uint64_t *crnlp_general_sgls_unlock(crnlp_general_sgls_lock_t* lock, uint8_t ind)
{
    crnlp_general_sgls_request_t *self;
    uint_fast8_t spin_loc;

    uint64_t *ret;
    uint64_t start, end;

    ret = (uint64_t *)malloc(sizeof(uint64_t) * 2);
    ret[0] = ret[1] = 0;

    start = get_cycles();

    self = &(lock->requests[ind]);

    self->done = 1;

    mb();

    self->needs_service = 1;
    while (self->needs_service != 0)
    {
        cpu_relax();
    }

    end = get_cycles();
    ret[0] += end - start;

    return ret;
}
